﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Configuration;

namespace DoBotConfig
{
    public partial class Configurator : Form
    {
        private bool loading = false;
        XmlDocument doc = new XmlDocument();

        private long saveTimerStarted = 0;

        public Configurator()
        {
            InitializeComponent();
            doc.Load("DoBotConfig.xml");
            PopulatePlayerName();
            pPlayerMaxs.Visible = bool.Parse(ConfigurationManager.AppSettings["pPlayerMaxs"]);
            pSkills.Visible = bool.Parse(ConfigurationManager.AppSettings["pSkills"]);
            pEnabled.Visible = bool.Parse(ConfigurationManager.AppSettings["pEnabled"]);
            pCooldown.Visible = bool.Parse(ConfigurationManager.AppSettings["pCooldown"]);
            pCastTime.Visible = bool.Parse(ConfigurationManager.AppSettings["pCastTime"]);
            pCauseGDC.Visible = bool.Parse(ConfigurationManager.AppSettings["pCauseGDC"]);
            pObey.Visible = bool.Parse(ConfigurationManager.AppSettings["pObey"]);
            pPriority.Visible = bool.Parse(ConfigurationManager.AppSettings["pPriority"]);
            pKeys.Visible = bool.Parse(ConfigurationManager.AppSettings["pKeys"]);
            pStates.Visible = bool.Parse(ConfigurationManager.AppSettings["pStates"]);
            pChanneled.Visible = bool.Parse(ConfigurationManager.AppSettings["pChanneled"]);
            pSelfCast.Visible = bool.Parse(ConfigurationManager.AppSettings["pSelfCast"]);
            pPlayer.Visible = bool.Parse(ConfigurationManager.AppSettings["pPlayer"]);
            pTarget.Visible = bool.Parse(ConfigurationManager.AppSettings["pTarget"]);
            pPet.Visible = bool.Parse(ConfigurationManager.AppSettings["pPet"]);
            pTargetBuf.Visible = bool.Parse(ConfigurationManager.AppSettings["pTargetBuf"]);
            pPlayerBuf.Visible = bool.Parse(ConfigurationManager.AppSettings["pPlayerBuf"]);
        }


        private void Save(object sender, EventArgs e)
        {
            SaveIt.Stop();
            SaveIt.Enabled = false;
            saveTimerStarted = 0;
            labelSaved.Visible = true;
            clearSaved.Enabled = true;
            clearSaved.Start();
            pbSave.Value = 0;
            SaveItProgress.Enabled = false;
            SaveItProgress.Stop();
            playerName.Enabled = true;
            skills.Enabled = true;

            string currentPlayer = playerName.Text;
            string currentSkill = skills.Text;
            UpdateXMLDocument();
            this.loading = true;
            PopulatePlayerName();
            PopulatePlayerSkillName();

            playerName.Text = currentPlayer;
            skills.Text = currentSkill;
            this.loading = false;
            SaveXMLDocument();
        }

#region GUI Events
        private void Changed(object sender, EventArgs e)
        {
            SetTimer();
        }

        private void ProgressBarClick(object sender, EventArgs e)
        {
            if (((MouseEventArgs)e).Button == System.Windows.Forms.MouseButtons.Left)
            {
                ((ProgressBar)sender).Value = ((MouseEventArgs)e).X;
            }
            if (((MouseEventArgs)e).Button == System.Windows.Forms.MouseButtons.Right)
            {
                ((ProgressBar)sender).Value = 0;
            }

            
            SetTimer();
        }
#endregion

#region Timers

        private void SetTimer()
        {
            if (loading == false)
            {
                playerName.Enabled = false;
                skills.Enabled = false;
                SaveIt.Stop();
                SaveIt.Enabled = false;
                SaveIt.Enabled = true;
                SaveIt.Start();
                saveTimerStarted = Environment.TickCount;
                SaveItProgress.Enabled = true;
                SaveItProgress.Start();
            }
        }

        private void SaveItProgress_Tick(object sender, EventArgs e)
        {
            if (saveTimerStarted != 0)
            {

                long ts = Environment.TickCount;
                int v = (int) (ts - saveTimerStarted) / 100;
                if (v < 0)
                {
                    v = 0;
                }
                if (v > 100)
                {
                    v = 100;
                }
                pbSave.Value = v;
            }
        }

        private void clearSaved_Tick(object sender, EventArgs e)
        {
            clearSaved.Stop();
            clearSaved.Enabled = false;
            labelSaved.Visible = false;
        }
#endregion


#region SaveXML
        public void SaveXMLDocument()
        {
            doc.Save("DoBotConfig.xml");
        }

        public void UpdateXMLDocument()
        {

            if(xml.ElementExists(doc, string.Format("/doc/Player[@name='{0}']", playerName.Text))){
                xml.SetElementNumericValue(doc, (int)this.playerMaxHealth.Value, string.Format("/doc/Player[@name='{0}']/MaxPlayerHealth", playerName.Text));
                xml.SetElementNumericValue(doc, (int)this.playerMaxMana.Value, string.Format("/doc/Player[@name='{0}']/MaxPlayerMana", playerName.Text));
                xml.SetElementNumericValue(doc, (int)this.playerMaxRage.Value, string.Format("/doc/Player[@name='{0}']/MaxPlayerRage", playerName.Text));
                xml.SetElementNumericValue(doc, (int)this.petMaxHealth.Value, string.Format("/doc/Player[@name='{0}']/MaxPetHealth", playerName.Text));

                if (xml.ElementExists(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']", playerName.Text, skills.Text)))
                {
                    xml.SetElementBoolValue(doc, skillEnabled.Checked, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/Enabled", playerName.Text, skills.Text));
                    xml.SetElementBoolValue(doc, pObeyGDC.Checked, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/ObeysGDC", playerName.Text, skills.Text));
                    xml.SetElementBoolValue(doc, skillChanneled.Checked, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/Channeled", playerName.Text, skills.Text));
                    xml.SetElementBoolValue(doc, skillSelfCast.Checked, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/SelfCast", playerName.Text, skills.Text));
                    xml.SetElementBoolValue(doc, skillValidatePlayer.Checked, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/ValidatePlayer", playerName.Text, skills.Text));
                    xml.SetElementBoolValue(doc, skillValidateTarget.Checked, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/ValidateTarget", playerName.Text, skills.Text));
                    xml.SetElementBoolValue(doc, skillValidatePet.Checked, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/ValidatePet", playerName.Text, skills.Text));

                    xml.SetElementNumericValue(doc, (int)skillCooldown.Value, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CoolDownSecs", playerName.Text, skills.Text));
                    xml.SetElementNumericValue(doc, (int)skillCausesGDC.Value, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CausesGDC", playerName.Text, skills.Text));
                    xml.SetElementNumericValue(doc, (int)skillCastTime.Value, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastTime", playerName.Text, skills.Text));
                    xml.SetElementNumericValue(doc, (int)skillPriority.Value, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/Priority", playerName.Text, skills.Text));
                    xml.SetElementStringValue(doc, skillKey.Text, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/Vkey", playerName.Text, skills.Text));
                    xml.SetElementStringValue(doc, skillKeyModifyer.Text, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/KeyModifyer", playerName.Text, skills.Text));

                    decimal v = 0;
                    if((decimal)(skillPlayerMinHealth.Value / 100.00) == 0) { v = 1; } else { v = (decimal)(skillPlayerMinHealth.Value / 100.00);}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowHealth", playerName.Text, skills.Text));
                    if((decimal)(skillPlayerMinMana.Value / 100.00) == 0) { v = 1; } else { v = (decimal)(skillPlayerMinMana.Value / 100.00);}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowMana", playerName.Text, skills.Text));
                    if((decimal)(skillPlayerMinRage.Value / 100.00) == 0) { v = 1; } else { v = (decimal)(skillPlayerMinRage.Value / 100.00);}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowRage", playerName.Text, skills.Text));
                    if((decimal)(skillTargetMinHealth.Value / 100.00) == 0) { v = 1; } else { v = (decimal)(skillTargetMinHealth.Value / 100.00);}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowTargetHealth", playerName.Text, skills.Text));
                    if((decimal)(skillPetMinHealth.Value / 100.00) == 0) { v = 1; } else { v = (decimal)(skillPetMinHealth.Value / 100.00);}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowPetHealth", playerName.Text, skills.Text));
                    
                    if((decimal)(1-(skillPlayerMaxHealth.Value / 100.00)) == 1) { v = 0; } else { v = (decimal)(1-(skillPlayerMaxHealth.Value / 100.00));}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAboveHealth", playerName.Text, skills.Text));
                    if((decimal)(1-(skillPlayerMaxMana.Value / 100.00)) == 1) { v = 0; } else { v = (decimal)(1-(skillPlayerMaxMana.Value / 100.00));}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAboveMana", playerName.Text, skills.Text));
                    if((decimal)(1-(skillPlayerMaxRage.Value / 100.00)) == 1) { v = 0; } else { v = (decimal)(1-(skillPlayerMaxRage.Value / 100.00));}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAboveRage", playerName.Text, skills.Text));
                    if((decimal)(1-(skillTargetMaxHealth.Value / 100.00)) == 1) { v = 0; } else { v = (decimal)(1-(skillTargetMaxHealth.Value / 100.00));}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAboveTargetHealth", playerName.Text, skills.Text));
                    if((decimal)(1-(skillPetMaxHealth.Value / 100.00)) == 1) { v = 0; } else { v = (decimal)(1-(skillPetMaxHealth.Value / 100.00));}
                    xml.SetElementDecimalValue(doc, v, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAbovePetHealth", playerName.Text, skills.Text));


                    xml.SetElementAttribute(doc, "hasnot",skillTargetHasNot.Checked.ToString(), string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/TargetBuf", playerName.Text, skills.Text));
                    xml.SetElementAttribute(doc, "hasnot", skillPlayerHasNot.Checked.ToString(), string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/PlayerBuf[@hasnot]", playerName.Text, skills.Text));
                    xml.SetElementStringValue(doc, skillTargetDeBuf.Text, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/TargetBuf", playerName.Text, skills.Text));
                    xml.SetElementStringValue(doc, skillPlayerDeBuf.Text, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/PlayerBuf", playerName.Text, skills.Text));


                    bool hasAttack =xml.HasElement(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States/State[.='Attack']", playerName.Text, skills.Text));
                    bool hasRest = xml.HasElement(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States/State[.='Rest']", playerName.Text, skills.Text));
                    bool hasBuf = xml.HasElement(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States/State[.='Buf']", playerName.Text, skills.Text));
                    if (hasAttack != skillStateAttack.Checked)
                    {
                        if (skillStateAttack.Checked)
                        {
                            xml.AddElement( doc, 
                                            string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States", playerName.Text, skills.Text),
                                            new XmlTextReader(new StringReader("<State>Attack</State>")));
                        }
                        else
                        {
                            xml.DeleteElement(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States/State[.='Attack']", playerName.Text, skills.Text));
                        }
                    }
                    if (hasBuf != skillStateBuf.Checked)
                    {
                        if (skillStateBuf.Checked)
                        {
                            xml.AddElement(doc,
                                          string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States", playerName.Text, skills.Text),
                                          new XmlTextReader(new StringReader("<State>Buf</State>")));
                        }
                        else
                        {
                            xml.DeleteElement(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States/State[.='Buf']", playerName.Text, skills.Text));
                        }
                    }
                    if (hasRest != skillStateRest.Checked)
                    {
                        if (skillStateRest.Checked)
                        {
                            xml.AddElement(doc,
                                            string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States", playerName.Text, skills.Text),
                                            new XmlTextReader(new StringReader("<State>Rest</State>")));
                        }
                        else
                        {
                            xml.DeleteElement(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States/State[.='Rest']", playerName.Text, skills.Text));
                        }
                    }

                }
                else
                {
                    if (skills.Text.Length > 0 && skills.Text.Trim() != "")
                    {
                        StringBuilder sb = new StringBuilder();
                        sb.Append(string.Format("<Skill name='{0}'>", skills.Text.Trim()));
                        sb.Append("<Enabled>False</Enabled>");
                        sb.Append("<CoolDownSecs>0</CoolDownSecs>");
                        sb.Append("<CastTime>0</CastTime>");
                        sb.Append("<Priority>0</Priority>");
                        sb.Append("<Vkey></Vkey>");
                        sb.Append("<KeyModifyer></KeyModifyer>");
                        sb.Append("<States>");
                        sb.Append("</States>");
                        sb.Append("<CastBelowHealth>0</CastBelowHealth>");
                        sb.Append("<CastBelowMana>0</CastBelowMana>");
                        sb.Append("<CastBelowRage>0</CastBelowRage>");
                        sb.Append("<CastAboveHealth>0</CastAboveHealth>");
                        sb.Append("<CastAboveMana>0</CastAboveMana>");
                        sb.Append("<CastAboveRage>0</CastAboveRage>");
                        sb.Append("<CastBelowTargetHealth>0</CastBelowTargetHealth>");
                        sb.Append("<CastAboveTargetHealth>0</CastAboveTargetHealth>");
                        sb.Append("<CausesGDC>0</CausesGDC>");
                        sb.Append("<ObeysGDC>True</ObeysGDC>");
                        sb.Append("<Channeled>False</Channeled>");
                        sb.Append("<CastBelowPetHealth>0</CastBelowPetHealth>");
                        sb.Append("<CastAbovePetHealth>0</CastAbovePetHealth>");
                        sb.Append("<SelfCast>False</SelfCast>");
                        sb.Append("<ValidatePlayer>False</ValidatePlayer>");
                        sb.Append("<ValidatePet>False</ValidatePet>");
                        sb.Append("<ValidateTarget>False</ValidateTarget>");
                        sb.Append("<TargetBuf hasnot='False'></TargetBuf>");
                        sb.Append("<PlayerBuf hasnot='False'></PlayerBuf>");
                        sb.Append("</Skill>");

                        xml.AddElement(doc,
                        string.Format("/doc/Player[@name='{0}']/Skills", playerName.Text),
                        new XmlTextReader(new StringReader(sb.ToString())));
                        UpdateXMLDocument();
                    }
                }

            }
            else
            {
                if (playerName.Text.Length > 0 && playerName.Text.Trim() != "")
                {
                    StringBuilder sb = new StringBuilder();
                    sb.Append(string.Format("<Player name='{0}'>", playerName.Text.Trim()));
                    sb.Append("<MaxPlayerHealth>0</MaxPlayerHealth>");
                    sb.Append("<MaxPlayerMana>0</MaxPlayerMana>");
                    sb.Append("<MaxPlayerRage>0</MaxPlayerRage>");
                    sb.Append("<MaxPetHealth>0</MaxPetHealth>");
                    sb.Append("<Skills>");
                    sb.Append("</Skills>");
                    sb.Append("</Player>");
                    xml.AddElement(doc, "/doc", new XmlTextReader(new StringReader(sb.ToString())));
                    UpdateXMLDocument();
                }
            }

        }
#endregion

#region LoadXML

        public void PopulatePlayerName()
        {
            playerName.Items.Clear();
            List<string> l = xml.GetElementConcatonatedAttribute(doc, "name", string.Format("/doc/Player[@name]", playerName.Text));
            foreach (string name in l)
            {
                playerName.Items.Add(name);
            }
        }

        public void PopulatePlayerSkillName()
        {
            skills.Items.Clear();
            List<string> l = xml.GetElementConcatonatedAttribute(doc, "name", string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name]", playerName.Text));
            foreach (string name in l)
            {
                skills.Items.Add(name);
            }
        }

        private void ClearPlayerSkill()
        {
            loading = true;
            skillEnabled.Checked = false;
            pObeyGDC.Checked = false;
            skillChanneled.Checked = false;
            skillSelfCast.Checked = false;
            skillValidatePlayer.Checked = false;
            skillValidatePet.Checked = false;
            skillValidateTarget.Checked = false;
            skillCooldown.Value = 0;
            skillCausesGDC.Value = 0;
            skillCastTime.Value = 0;
            skillPriority.Value = 0;
            skillKey.Text = "";
            skillKeyModifyer.Text = "";

            skillStateAttack.Checked = false;
            skillStateRest.Checked = false;
            skillStateBuf.Checked = false;

            skillPlayerMinHealth.Value = 0;
            skillPlayerMinMana.Value = 0;
            skillPlayerMinRage.Value = 0;
            skillPlayerMaxHealth.Value = 0;
            skillPlayerMaxMana.Value = 0;
            skillPlayerMaxRage.Value = 0;
            skillTargetMinHealth.Value = 0;
            skillTargetMaxHealth.Value = 0;
            skillPetMinHealth.Value = 0;
            skillPetMaxHealth.Value = 0;
            skills.Text = "";

            skillTargetHasNot.Checked = false;
            skillPlayerHasNot.Checked = false;
            skillTargetDeBuf.Text = "";
            skillPlayerDeBuf.Text = "";

            loading = false;
        }

        private void LoadPlayerSkill(object sender, EventArgs e)
        {
            loading = true;
            skillEnabled.Checked = xml.GetElementBoolValue(doc,string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/Enabled", playerName.Text, skills.Text));
            pObeyGDC.Checked = xml.GetElementBoolValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/ObeysGDC", playerName.Text, skills.Text));
            skillChanneled.Checked = xml.GetElementBoolValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/Channeled", playerName.Text, skills.Text));
            skillSelfCast.Checked = xml.GetElementBoolValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/SelfCast", playerName.Text, skills.Text));
            skillValidatePlayer.Checked = xml.GetElementBoolValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/ValidatePlayer", playerName.Text, skills.Text));
            skillValidateTarget.Checked = xml.GetElementBoolValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/ValidateTarget", playerName.Text, skills.Text));
            skillValidatePet.Checked = xml.GetElementBoolValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/ValidatePet", playerName.Text, skills.Text));

            skillCooldown.Value = xml.GetElementNumericValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CoolDownSecs", playerName.Text, skills.Text));
            skillCausesGDC.Value = xml.GetElementNumericValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CausesGDC", playerName.Text, skills.Text));
            skillCastTime.Value = xml.GetElementNumericValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastTime", playerName.Text, skills.Text));
            skillPriority.Value = xml.GetElementNumericValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/Priority", playerName.Text, skills.Text));
            skillKey.Text = xml.GetElementStringValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/Vkey", playerName.Text, skills.Text));
            skillKeyModifyer.Text = xml.GetElementStringValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/KeyModifyer", playerName.Text, skills.Text));


            List<string> l = xml.GetElementConcatonatedValues(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/States/State", playerName.Text, skills.Text));
            skillStateAttack.Checked = l.Contains("Attack");
            skillStateRest.Checked = l.Contains("Rest");
            skillStateBuf.Checked = l.Contains("Buf");

            skillPlayerMinHealth.Value = (int)(100 * xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowHealth", playerName.Text, skills.Text)));
            if (skillPlayerMinHealth.Value == 100) { skillPlayerMinHealth.Value = 0; }
            skillPlayerMinMana.Value = (int)(100 * xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowMana", playerName.Text, skills.Text)));
            if (skillPlayerMinMana.Value == 100) { skillPlayerMinMana.Value = 0; }
            skillPlayerMinRage.Value = (int)(100 * xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowRage", playerName.Text, skills.Text)));
            if (skillPlayerMinRage.Value == 100) { skillPlayerMinRage.Value = 0; }
            skillTargetMinHealth.Value = (int)(100 * xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowTargetHealth", playerName.Text, skills.Text)));
            if (skillTargetMinHealth.Value == 100) { skillTargetMinHealth.Value = 0; }
            skillPetMinHealth.Value = (int)(100 * xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastBelowPetHealth", playerName.Text, skills.Text)));
            if (skillPetMinHealth.Value == 100) { skillPetMinHealth.Value = 0; }

            skillPlayerMaxHealth.Value = (int)(100 * (1 - xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAboveHealth", playerName.Text, skills.Text))));
            if (skillPlayerMaxHealth.Value == 100) { skillPlayerMaxHealth.Value = 0; }
            skillPlayerMaxMana.Value = (int)(100 * (1 - xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAboveMana", playerName.Text, skills.Text))));
            if (skillPlayerMaxMana.Value == 100) { skillPlayerMaxMana.Value = 0; }
            skillPlayerMaxRage.Value = (int)(100 * (1 - xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAboveRage", playerName.Text, skills.Text))));
            if (skillPlayerMaxRage.Value == 100) { skillPlayerMaxRage.Value = 0; }
            skillTargetMaxHealth.Value = (int)(100 * (1-xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAboveTargetHealth", playerName.Text, skills.Text))));
            if (skillTargetMaxHealth.Value == 100) { skillTargetMaxHealth.Value = 0; }
            skillPetMaxHealth.Value = (int)(100 * (1 - xml.GetElementDecimalValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/CastAbovePetHealth", playerName.Text, skills.Text))));
            if (skillPetMaxHealth.Value == 100) { skillPetMaxHealth.Value = 0; }

            skillTargetHasNot.Checked = bool.Parse(xml.GetElementAttribute(doc,"hasnot", string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/TargetBuf", playerName.Text, skills.Text)));
            skillPlayerHasNot.Checked = bool.Parse(xml.GetElementAttribute(doc, "hasnot", string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/PlayerBuf", playerName.Text, skills.Text)));
            skillTargetDeBuf.Text = xml.GetElementStringValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/TargetBuf", playerName.Text, skills.Text));
            skillPlayerDeBuf.Text = xml.GetElementStringValue(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']/PlayerBuf", playerName.Text, skills.Text));
            loading = false;
        }

        private void ClearPlayer()
        {
            loading = true;
            this.playerMaxHealth.Value = 0;
            this.playerMaxMana.Value = 0;
            this.playerMaxRage.Value = 0;
            this.petMaxHealth.Value = 0;
            playerName.Text = "";
            ClearPlayerSkill();
            loading = false;
        }

        private void LoadPlayer(object sender, EventArgs e)
        {
            loading = true;
            this.playerMaxHealth.Value = xml.GetElementNumericValue(doc, string.Format("/doc/Player[@name='{0}']/MaxPlayerHealth", playerName.Text));
            this.playerMaxMana.Value = xml.GetElementNumericValue(doc, string.Format("/doc/Player[@name='{0}']/MaxPlayerMana", playerName.Text));
            this.playerMaxRage.Value = xml.GetElementNumericValue(doc, string.Format("/doc/Player[@name='{0}']/MaxPlayerRage", playerName.Text));
            this.petMaxHealth.Value = xml.GetElementNumericValue(doc, string.Format("/doc/Player[@name='{0}']/MaxPetHealth", playerName.Text));
            PopulatePlayerSkillName();
            loading = false;
        }
#endregion

        private void deleteSkill_Click(object sender, EventArgs e)
        {
            xml.DeleteElement(doc, string.Format("/doc/Player[@name='{0}']/Skills/Skill[@name='{1}']", playerName.Text, skills.Text));
            SaveXMLDocument();
            ClearPlayerSkill();
            PopulatePlayerSkillName();
        }

        private void deletePlayer_Click(object sender, EventArgs e)
        {
            xml.DeleteElement(doc, string.Format("/doc/Player[@name='{0}']", playerName.Text));
            SaveXMLDocument();
            ClearPlayer();
            PopulatePlayerName();

        }

        private void pbSave_Click(object sender, EventArgs e)
        {

        }

        private void skillValidatePlayer_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void skillValidateTarget_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void skillValidatePet_CheckedChanged(object sender, EventArgs e)
        {

        }

    }
}
