﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.XPath;

namespace DoBotConfig
{
    class xml
    {

        public static bool ElementExists(XmlDocument doc, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public static string GetElementStringValue(XmlDocument doc, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                return iterator.Current.Value;
            }
            else
            {
                return "";
            }
        }

        public static bool SetElementStringValue(XmlDocument doc, string value, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                iterator.Current.SetValue (value);
                return true;
            }
            else
            {
                return false;
            }
        }

        public static bool GetElementBoolValue(XmlDocument doc, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                return bool.Parse(iterator.Current.Value);
            }
            else
            {
                return false;
            }
        }

        public static bool SetElementBoolValue(XmlDocument doc, bool value, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                iterator.Current.SetValue(value.ToString());
                return true;
            }
            else
            {
                return false;
            }
        }


        public static decimal GetElementDecimalValue(XmlDocument doc, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                return decimal.Parse(iterator.Current.Value);
            }
            else
            {
                return 0;
            }
        }

        public static bool SetElementDecimalValue(XmlDocument doc, decimal value, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                iterator.Current.SetValue(string.Format("{0:0.##}", value));
                return true;
            }
            else
            {
                return false;
            }
        }

        public static int GetElementNumericValue(XmlDocument doc, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                return int.Parse(iterator.Current.Value);
            }
            else
            {
                return 0;
            }
        }

        public static bool SetElementNumericValue(XmlDocument doc, int value, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                iterator.Current.SetValue(value.ToString());
                return true;
            }
            else
            {
                return false;
            }
        }

        public static List<string> GetElementConcatonatedValues(XmlDocument doc, string xpath)
        {
            List<string> retval = new List<string>();

            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            while (iterator.MoveNext())
            {
                retval.Add(iterator.Current.Value);
            }
            return retval;
        }

        public static List<string> GetElementConcatonatedAttribute(XmlDocument doc, string attribute, string xpath)
        {
            List<string> retval = new List<string>();

            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            while (iterator.MoveNext())
            {
                retval.Add(iterator.Current.GetAttribute(attribute, ""));
            }
            return retval;
        }


        public static string GetElementAttribute(XmlDocument doc, string attribute, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            while (iterator.MoveNext())
            {
                return iterator.Current.GetAttribute(attribute, "");
            }
            return "";
        }

        public static void SetElementAttribute(XmlDocument doc, string attribute, string value, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            while (iterator.MoveNext())
            {
                iterator.Current.MoveToAttribute(attribute, "");
                iterator.Current.SetValue(value);
            }
        }
        public static bool HasElement(XmlDocument doc, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public static void DeleteElement(XmlDocument doc, string xpath)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                iterator.Current.DeleteSelf();
            }
        }

        public static void AddElement(XmlDocument doc, string xpath,XmlReader docFragment)
        {
            XPathNavigator nav = doc.CreateNavigator();
            XPathExpression expr = nav.Compile(xpath);
            XPathNodeIterator iterator = nav.Select(expr);

            if (iterator.MoveNext() == true)
            {
                iterator.Current.AppendChild(docFragment);
            }
        }
    }
}
